package com.company.tokenrevokeexample.screen.tokenrevoke;

import io.jmix.core.security.CurrentAuthentication;
import io.jmix.securityoauth2.impl.TokenRevoker;
import io.jmix.ui.Notifications;
import io.jmix.ui.component.Button;
import io.jmix.ui.screen.Screen;
import io.jmix.ui.screen.Subscribe;
import io.jmix.ui.screen.UiController;
import io.jmix.ui.screen.UiDescriptor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.OAuth2Request;
import org.springframework.security.oauth2.provider.token.DefaultTokenServices;
import org.springframework.security.oauth2.provider.token.TokenStore;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

@UiController("TokenRevokeScreen")
@UiDescriptor("token-revoke-screen.xml")
public class TokenRevokeScreen extends Screen {
    @Autowired
    private Notifications notifications;
    @Autowired
    private TokenStore tokenStore;
    @Autowired
    private CurrentAuthentication currentAuthentication;
    @Autowired
    private DefaultTokenServices defaultTokenServices;
    @Autowired
    private TokenRevoker tokenRevoker;

    @Subscribe("showTokens")
    public void onShowTokensClick(final Button.ClickEvent event) {
        Collection<OAuth2AccessToken> clientTokens = tokenStore.findTokensByClientId("client");
        StringBuilder tokens = new StringBuilder();
        for (OAuth2AccessToken token : clientTokens) {
            tokens.append(token.getValue()).append("\n");
        }
        notifications.create()
                .withCaption("Tokens count: " + clientTokens.size())
                .withDescription(tokens.toString())
                .show();    
    }

    @Subscribe("generateToken")
    public void onGenerateTokenClick(final Button.ClickEvent event) {
        notifications.create().withCaption("Token [" + createToken().getValue() + "] created!").show();
    }

    private OAuth2AccessToken createToken() {
        Map<String, String> params = new HashMap<>();
        params.put("client_secret", "secret");
        params.put("user_name", "admin");
        params.put("password", "admin");

        OAuth2Request storedRequest = new OAuth2Request(
                params, "client", null, true, Set.of("read"), null, null, null, null);
        OAuth2Authentication authentication = new OAuth2Authentication(
                storedRequest, currentAuthentication.getAuthentication());

        return defaultTokenServices.createAccessToken(authentication);
    }

    @Subscribe("generateTokenWithRevoke")
    public void onGenerateTokenWithRevokeClick(final Button.ClickEvent event) {
        OAuth2AccessToken newToken = createToken();

        Collection<OAuth2AccessToken> clientTokens = tokenStore.findTokensByClientId("client");
        for (OAuth2AccessToken oldToken : clientTokens) {
            if (!newToken.getValue().equals(oldToken.getValue())) {
                tokenRevoker.revokeAccessToken(oldToken.getValue());
            }
        }

        notifications.create().withCaption("Token [" + newToken.getValue() + "] created!").show();
    }
}